@@ -16,6 +16,7 @@
 // under the License.
 
 #include <cmath>
+#include <concepts>
 #include <functional>
 #include <memory>
 #include <string>
@@ -32,6 +33,7 @@
 #include "arrow/compute/kernels/common_internal.h"
 #include "arrow/compute/kernels/hash_aggregate_internal.h"
 #include "arrow/compute/kernels/util_internal.h"
+#include "arrow/compute/registry_internal.h"
 #include "arrow/compute/row/grouper.h"
 #include "arrow/compute/row/row_encoder_internal.h"
 #include "arrow/record_batch.h"
@@ -269,52 +271,61 @@ struct GroupedCountImpl : public GroupedAggregator {
 // ----------------------------------------------------------------------
 // MinMax implementation
 
+// XXX: Consider making these concepts complete and moving to public header.
+
+template <typename T>
+concept CBooleanConcept = std::same_as<T, bool>;
+
+// XXX: Ideally we want to have std::floating_point<Float16> = true.
+// Some older standard library implementations (e.g., macOS 11.x libc++) have partial
+// C++20 concepts support with std::same_as but lack std::floating_point.
+#if defined(__cpp_lib_concepts) && __cpp_lib_concepts >= 202002L
+template <typename T>
+concept CFloatingPointConcept = std::floating_point<T> || std::same_as<T, util::Float16>;
+#else
+template <typename T>
+concept CFloatingPointConcept =
+    std::is_floating_point_v<T> || std::same_as<T, util::Float16>;
+#endif
+
+template <typename T>
+concept CDecimalConcept = std::same_as<T, Decimal32> || std::same_as<T, Decimal64> ||
+    std::same_as<T, Decimal128> || std::same_as<T, Decimal256>;
+
 template <typename CType>
 struct AntiExtrema {
   static constexpr CType anti_min() { return std::numeric_limits<CType>::max(); }
   static constexpr CType anti_max() { return std::numeric_limits<CType>::min(); }
 };
 
-template <>
-struct AntiExtrema<bool> {
-  static constexpr bool anti_min() { return true; }
-  static constexpr bool anti_max() { return false; }
+template <CBooleanConcept CType>
+struct AntiExtrema<CType> {
+  static constexpr CType anti_min() { return true; }
+  static constexpr CType anti_max() { return false; }
 };
 
-template <>
-struct AntiExtrema<float> {
-  static constexpr float anti_min() { return std::numeric_limits<float>::infinity(); }
-  static constexpr float anti_max() { return -std::numeric_limits<float>::infinity(); }
+template <CFloatingPointConcept CType>
+struct AntiExtrema<CType> {
+  static constexpr CType anti_min() { return std::numeric_limits<CType>::quiet_NaN(); }
+  static constexpr CType anti_max() { return std::numeric_limits<CType>::quiet_NaN(); }
 };
 
-template <>
-struct AntiExtrema<double> {
-  static constexpr double anti_min() { return std::numeric_limits<double>::infinity(); }
-  static constexpr double anti_max() { return -std::numeric_limits<double>::infinity(); }
+template <CDecimalConcept CType>
+struct AntiExtrema<CType> {
+  static constexpr CType anti_min() { return CType::GetMaxSentinel(); }
+  static constexpr CType anti_max() { return CType::GetMinSentinel(); }
 };
 
-template <>
-struct AntiExtrema<Decimal32> {
-  static constexpr Decimal32 anti_min() { return BasicDecimal32::GetMaxSentinel(); }
-  static constexpr Decimal32 anti_max() { return BasicDecimal32::GetMinSentinel(); }
+template <typename CType>
+struct MinMaxOp {
+  static constexpr CType min(CType a, CType b) { return std::min(a, b); }
+  static constexpr CType max(CType a, CType b) { return std::max(a, b); }
 };
 
-template <>
-struct AntiExtrema<Decimal64> {
-  static constexpr Decimal64 anti_min() { return BasicDecimal64::GetMaxSentinel(); }
-  static constexpr Decimal64 anti_max() { return BasicDecimal64::GetMinSentinel(); }
-};
-
-template <>
-struct AntiExtrema<Decimal128> {
-  static constexpr Decimal128 anti_min() { return BasicDecimal128::GetMaxSentinel(); }
-  static constexpr Decimal128 anti_max() { return BasicDecimal128::GetMinSentinel(); }
-};
-
-template <>
-struct AntiExtrema<Decimal256> {
-  static constexpr Decimal256 anti_min() { return BasicDecimal256::GetMaxSentinel(); }
-  static constexpr Decimal256 anti_max() { return BasicDecimal256::GetMinSentinel(); }
+template <CFloatingPointConcept CType>
+struct MinMaxOp<CType> {
+  static constexpr CType min(CType a, CType b) { return std::fmin(a, b); }
+  static constexpr CType max(CType a, CType b) { return std::fmax(a, b); }
 };
 
 template <typename Type, typename Enable = void>
@@ -351,8 +362,8 @@ struct GroupedMinMaxImpl final : public GroupedAggregator {
     VisitGroupedValues<Type>(
         batch,
         [&](uint32_t g, CType val) {
-          GetSet::Set(raw_mins, g, std::min(GetSet::Get(raw_mins, g), val));
-          GetSet::Set(raw_maxes, g, std::max(GetSet::Get(raw_maxes, g), val));
+          GetSet::Set(raw_mins, g, MinMaxOp<CType>::min(GetSet::Get(raw_mins, g), val));
+          GetSet::Set(raw_maxes, g, MinMaxOp<CType>::max(GetSet::Get(raw_maxes, g), val));
           bit_util::SetBit(has_values_.mutable_data(), g);
         },
         [&](uint32_t g) { bit_util::SetBit(has_nulls_.mutable_data(), g); });
@@ -372,12 +383,12 @@ struct GroupedMinMaxImpl final : public GroupedAggregator {
     auto g = group_id_mapping.GetValues<uint32_t>(1);
     for (uint32_t other_g = 0; static_cast<int64_t>(other_g) < group_id_mapping.length;
          ++other_g, ++g) {
-      GetSet::Set(
-          raw_mins, *g,
-          std::min(GetSet::Get(raw_mins, *g), GetSet::Get(other_raw_mins, other_g)));
-      GetSet::Set(
-          raw_maxes, *g,
-          std::max(GetSet::Get(raw_maxes, *g), GetSet::Get(other_raw_maxes, other_g)));
+      GetSet::Set(raw_mins, *g,
+                  MinMaxOp<CType>::min(GetSet::Get(raw_mins, *g),
+                                       GetSet::Get(other_raw_mins, other_g)));
+      GetSet::Set(raw_maxes, *g,
+                  MinMaxOp<CType>::max(GetSet::Get(raw_maxes, *g),
+                                       GetSet::Get(other_raw_maxes, other_g)));
 
       if (bit_util::GetBit(other->has_values_.data(), other_g)) {
         bit_util::SetBit(has_values_.mutable_data(), *g);
